///////////////////////////////////////////////////////////////////////////////
//===========================================================================//
// COPYRIGHT:                                                                //
// ----------                                                                //
// Copyright (C) International Business Machines Corp., 1999.                //
//                                                                           //
// DISCLAIMER OF WARRANTIES:                                                 //
// -------------------------                                                 //
// The following [enclosed] code is sample code created by IBM               //
// Corporation.  This sample code is not part of any standard IBM            //
// product and is provided to you solely for the purpose of                  //
// assisting you in the development of your applications.  The               //
// code is provided "AS IS", without warranty of any kind.  IBM              //
// shall not be liable for any damages arising out of your use of            //
// the sample code, even if they have been advised of the                    //
// possibility of such damages.                                              //
//===========================================================================//
//////////////////////////////////////////////////////////////////////////////
//
//         main.c : Main program for Calculator Control   
//
///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifdef DEBUGMSGS
#define RECOMSGS 1
#endif

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/DialogS.h>
#include <stdio.h>


// variables global to this file

static Display *dspG;
static Widget mainTopShellG;
static Widget popupShellG;
static Window root;
static Window parent;
static Window child;
static Window  popup_window = 0;
static Widget displayLabel;
static Boolean CalcFound = False;
static XtAppContext theAppG;
static XtIntervalId xtIntrvlIdForClose = 0;


static char labelString[255] = "   << Click here to turn on the microphone";

#define CLOSE_TIME 500

//defines
#define SHELL_MIN_WIDTH      600
#define SHELL_MAX_WIDTH      600
#define SHELL_MIN_HEIGHT     200
#define SHELL_MAX_HEIGHT     200

Boolean GetWindowInfoRecursive (  Display * display, Window child,  Window  * the_child);
Boolean GetCalcWindow ( Display * display, Window root,  Window *child, Window *parent);
static void CloseControlCB (Widget w, XtPointer client_data, XtPointer call_data);
void DisplaySendMessage (Widget displaylabel, char *msg, Window child, Window parent, Window root, Boolean sendFlag);
void CalCtrlUsage(void);




/*----------------------------------------------------*/
/* All of the speech-related things (prototypes,      */
/* data types, etc)                                   */
/*----------------------------------------------------*/
#include <smapi.h>
#include <vtbnfc.h>




#ifdef DEBUGMSGS
/*----------------------------------------------------*/
/* A macro to check the return code from various Sm   */
/* calls (makes the code below look cleaner)          */
/*----------------------------------------------------*/
#define CheckSmRC(fn)                                  \
{                                                      \
  int rc;                                              \
                                                       \
  SmGetRc ( reply, & rc );                             \
                                                       \
  sprintf ( buffer, "%s: rc = %d", fn, rc );           \
                                                       \
  fprintf (stderr, "%s\n", buffer);                    \
                                                       \
  if ( rc != SM_RC_OK ) return ( SM_RC_OK );           \
}
#else
#define CheckSmRC(fn)
#endif
/*----------------------------------------------------*/
/* A couple of important values that get passed about */
/*----------------------------------------------------*/
typedef struct
{
  void * top_level;
  int x_input_id;
} MyClientData;

/*----------------------------------------------------------*/
/* A couple of callbacks - for the exit and mic buttons     */
/*----------------------------------------------------------*/
static void CloseUpShop  ( Widget, XtPointer, XtPointer );
static void ToggleMic    ( Widget, XtPointer, XtPointer );

static void ConnectStuff ( MyClientData * );
static void TurnMicOn    ( );
static void TurnMicOff   ( );


static Widget micButton;
static Widget userLabel;
static Pixmap btnPixmapOff;
static Pixmap btnPixmapOn;
static int    mic_state = 0;
static void *fsg = NULL;

static char   buffer   [ 255 ];

/*----------------------------------------------------------*/
/* Use whatever default speech user info there is..         */
/*----------------------------------------------------------*/
static char   userid   [ 80 ] = SM_USE_CURRENT;
static char   enrollid [ 80 ] = SM_USE_CURRENT;
static char   taskid   [ 80 ] = SM_USE_CURRENT;



/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
void SetButtonLabel ( Boolean Flag )
{
  char nullLabelString[1];

  nullLabelString[0] = '\0';

  if (Flag)
  { 
     if (CalcFound)
     {
        XtVaSetValues (micButton, XmNlabelPixmap, btnPixmapOn, NULL );
        DisplaySendMessage (displayLabel, nullLabelString, child, parent, root, False);
     }
  }
  else
  {
     if (CalcFound)
     {
       DisplaySendMessage (displayLabel, labelString, child, parent, root, False);
       XtVaSetValues (micButton, XmNlabelPixmap, btnPixmapOff, NULL );
     }
  }
}

/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static int myNotifier(int socket_handle, int (*recv_fn)(), void *recv_data,
  void *client_data)
{
  MyClientData *myData = (MyClientData *) client_data;
  XtAppContext app_context;
  if (recv_fn == NULL) {
    XtRemoveInput( myData->x_input_id );
    return SM_RC_OK;
  }
  app_context = XtWidgetToApplicationContext( myData->top_level );
  myData->x_input_id = XtAppAddInput(app_context, socket_handle,
        (void *) XtInputReadMask,
        (void (*)(void *,int *, unsigned long *)) recv_fn, recv_data);
  return SM_RC_OK;
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void ConnectStuff ( MyClientData * MyData )
{
  static int first = TRUE;
  int        rc;
  int        smc;
  SmArg      smargs [ 30 ];
  char       * cp;
  SM_MSG reply;
  /*-------------------------------------------------------------------*/
  /* These callbacks handle the various messages that the speech       */
  /* engine might be sending back                                      */
  /*-------------------------------------------------------------------*/
  SmHandler ConnectCB     ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler DisconnectCB  ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler SetCB         ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler MicOnCB       ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler MicOffCB      ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler DefineGrammarCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler DefineVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler EnableVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler GetNextWordCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler RecoWordCB    ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler RecoPhraseCB  ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler UtteranceCB   ( SM_MSG reply, caddr_t client, caddr_t call_data );


  if ( first )
  {
    smc = 0;
    SmSetArg ( smargs [ smc ], SmNapplicationName,      "CalCtrl" );  smc++;
    SmSetArg ( smargs [ smc ], SmNexternalNotifier,     myNotifier ); smc++;
    SmSetArg ( smargs [ smc ], SmNexternalNotifierData, MyData );     smc++;
   

    /*-----------------------------------------------------------------*/
    /* The call to SmOpen initializes any data that's inside of libSm  */
    /*-----------------------------------------------------------------*/
    rc = SmOpen ( smc, smargs );

    if ( rc != SM_RC_OK )
    {
       CheckSmRC("SmOpen() failed");
       return;
    }

    /*-----------------------------------------------------------------*/
    /* Add the callbacks to catch the messages coming back from the    */
    /* reco engine                                                     */
    /*-----------------------------------------------------------------*/
    SmAddCallback ( SmNconnectCallback,             ConnectCB,       NULL );
    SmAddCallback ( SmNdisconnectCallback,          DisconnectCB,    NULL );
    SmAddCallback ( SmNsetCallback,                 SetCB,           NULL );
    SmAddCallback ( SmNmicOnCallback,               MicOnCB,         NULL );
    SmAddCallback ( SmNmicOffCallback,              MicOffCB,        NULL );
    SmAddCallback ( SmNenableVocabCallback,         EnableVocabCB,   NULL );
    SmAddCallback ( SmNdefineVocabCallback,         DefineVocabCB,   NULL );
    SmAddCallback ( SmNdefineGrammarCallback,       DefineGrammarCB, NULL );
    SmAddCallback ( SmNrecognizeNextWordCallback,   GetNextWordCB,   NULL );
    SmAddCallback ( SmNrecognizedWordCallback,      RecoWordCB,      NULL );
    SmAddCallback ( SmNrecognizedPhraseCallback,    RecoPhraseCB,    (void *)MyData);
    SmAddCallback ( SmNutteranceCompletedCallback,  UtteranceCB,     NULL );

    first = FALSE;
  }

  /*-----------------------------------------------------------------*/
  /* Now connect to the engine (asynchronously, which means that     */
  /* the ConnectCB will get the results)                             */
  /*-----------------------------------------------------------------*/
  smc = 0;
  SmSetArg ( smargs [ smc ], SmNuserId,       userid    );  smc++;
  SmSetArg ( smargs [ smc ], SmNenrollId,     enrollid  );  smc++;
  SmSetArg ( smargs [ smc ], SmNtask,         taskid    );  smc++;
  SmSetArg ( smargs [ smc ], SmNrecognize,    TRUE      );  smc++;
  SmSetArg ( smargs [ smc ], SmNoverrideLock, TRUE      );  smc++;

  rc = SmConnect ( smc, smargs, SmAsynchronous );

  CheckSmRC ("ConnectStuff: SmConnect()");
}


/*---------------------------------------------------------------------*/
/* The user pressed the big microphone button - change the mic to      */
/* whichever state it's not in now (on -> off, off -> on)              */
/*---------------------------------------------------------------------*/
static void ToggleMic ( Widget w, XtPointer a, XtPointer b )
{
  if ( ! mic_state )
  {
    TurnMicOn ( );
  }
  else
  {
    TurnMicOff ( );
  }
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void TurnMicOn ( )
{
  SM_MSG  reply;
  int     rc;

  /*-------------------------------------------------------------------*/
  /* Send a request to tell the engine to turn the mic on.  The reply  */
  /* comes back to the MicOnCB                                         */
  /*-------------------------------------------------------------------*/
  if (CalcFound)
  {
     rc = SmMicOn ( SmAsynchronous );
     CheckSmRC ("TurnMicOn: SmMicOn()");
  }
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void TurnMicOff ( )
{
  SM_MSG  reply;
  int     rc;


  /*-------------------------------------------------------------------*/
  /* Same as above, but turning the mic off...                         */
  /*-------------------------------------------------------------------*/
  rc = SmMicOff( SmAsynchronous );

  CheckSmRC ("TurnMicOff: SmMicOff()");

}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static int DoSimpleGrammar ( )
{
  int    rc;
  char   grammar [ 512 ];
  char   currdir [ 512 ];
  char * cp;
  SM_MSG reply;
  /*-------------------------------------------------------------------*/
  /* The grammar that we're using should be in the same directory as   */
  /* the program was run from - the reco engine expects a complete     */
  /* path to it (or a path relative to where the engine is run from)   */
  /* so we build it using getcwd()                                     */
  /*-------------------------------------------------------------------*/
  cp = ( char * ) getcwd ( currdir, sizeof ( currdir ) - 1 );

  sprintf ( grammar, "%s/calexpression.fsg", currdir );

  /*-------------------------------------------------------------------*/
  /* And now just tell the engine where the grammar is and let it do   */
  /* its thing.  The DefineGrammarCB will get the results back..       */
  /*-------------------------------------------------------------------*/
  rc = SmDefineGrammar ( "NumberGrammar", grammar, 0, SmAsynchronous );

#ifdef RECOMSGS  //@ABP
  sprintf ( buffer, "%s: rc = %d", "DoSimpleGrammar: SmDefineGrammar", rc );
  fprintf (stderr, "%s\n", buffer);
#endif

  return ( rc );
}
/*---------------------------------------------------------------------*/
/*            Do Load FST grammar file                                 */
/*---------------------------------------------------------------------*/
static int DoLoadFst()
{
  char   grammar [ 512 ];
  char   currdir [ 512 ];
  char * cp;

  /*-------------------------------------------------------------------*/
  /* The grammar that we're using should be in the same directory as   */
  /* the program was run from - the reco engine expects a complete     */
  /* path to it (or a path relative to where the engine is run from)   */
  /* so we build it using getcwd()                                     */
  /*-------------------------------------------------------------------*/
  cp = ( char * ) getcwd ( currdir, sizeof ( currdir ) - 1 );

  sprintf ( grammar, "%s/calexpression.fst", currdir );
  VtLoadFSG(grammar, &fsg);
}

/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void CloseUpShop ( Widget w, XtPointer a, XtPointer b )
{
  SM_MSG reply;
  /*-------------------------------------------------------------------*/
  /* We're outta here..                                                */
  /*-------------------------------------------------------------------*/
  SmClose();
  exit ( 0 );
}



/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler ConnectCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  int    rc;
  char * cp;
  char *UserName;
  XmString xmstrUserName;

  CheckSmRC("ConnectCB");


  rc = SmQueryDefault (SM_DEFAULT_USERID, &reply);
  CheckSmRC("SmQueryDefault");

  rc = SmGetUserId (reply, &UserName); 

  xmstrUserName = XmStringCreateLtoR ( UserName, XmSTRING_DEFAULT_CHARSET );
  XtVaSetValues ( userLabel, XmNlabelString, xmstrUserName, NULL );
  XmStringFree (xmstrUserName);

  CheckSmRC("SmGetUserId");


  /*-------------------------------------------------------------------*/
  /* We're "here", so we're connected to the engine, so now we define  */
  /* the grammar that we want the engine to use, and make the mic      */
  /* sensitive (so it can be pressed)                                  */
  /*-------------------------------------------------------------------*/

  DoSimpleGrammar ( );
  DoLoadFst();

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler DisconnectCB  ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler SetCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler MicOnCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("MicOnCB");

  /*-------------------------------------------------------------------*/
  /* The mic got turned on, sp change its label accordingly            */
  /*-------------------------------------------------------------------*/
  SetButtonLabel ( True );

  mic_state = 1;

  /*-------------------------------------------------------------------*/
  /* VERY IMPORTANT - this tells the recognizer to 'go' (ie. start     */
  /* capturing the audio and processing it)                            */
  /*-------------------------------------------------------------------*/
  SmRecognizeNextWord ( SmAsynchronous );

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler MicOffCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("MicOffCB");

  /*-------------------------------------------------------------------*/
  /* Mic's off, just change the button label..                         */
  /*-------------------------------------------------------------------*/
  SetButtonLabel ( False );

  XtVaSetValues (micButton, XmNlabelPixmap, btnPixmapOff, NULL );

  mic_state = 0;

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler EnableVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("EnableVocabCB");

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler DefineVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  char          * vocab;
  int             rc;
  int             i;
  SM_VOCWORD    * missing;
  unsigned long   num_missing;

  CheckSmRC("DefineVocabCB");

  SmGetVocabName ( reply, & vocab );


  /*-------------------------------------------------------------------*/
  /* Check to see if any of the words from the vocabulary are missing  */
  /* from the recognizers pool(s)                                      */
  /*-------------------------------------------------------------------*/
  rc = SmGetVocWords ( reply, & num_missing, & missing );

  CheckSmRC ("DefineVocabCB SmGetVocWords()");

#ifdef MISSINGVOCABS
  for ( i = 0 ; i < ( int ) num_missing; i++ )
  {

    sprintf ( buffer, "DefineVocabCB: word [ %d ] = '%s'",
              i, missing [ i ].spelling );
    fprintf (stderr,  buffer );

  }
#endif
  /*-------------------------------------------------------------------*/
  /* Enable the vocabulary (tells the recognizer to listen for words   */
  /* from it)                                                          */
  /*-------------------------------------------------------------------*/
  SmEnableVocab ( vocab, SmAsynchronous );

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler DefineGrammarCB ( SM_MSG reply, caddr_t client, caddr_t caller )
{
  int             rc;
  int             i;
  char            buffer [ 512 ];
  char          * vocab_name;
  SM_VOCWORD    * missing;
  unsigned long   num_missing;


  /*--------------------------------------------------------------------*/
  /* One of the reasons a grammar can fail to be defined is if the fsg  */
  /* has words that we don't have pronunciations for.  If this happens, */
  /* the missing words are included in the reply structure.             */
  /*--------------------------------------------------------------------*/
  SmGetRc ( reply, & rc );

  if ( rc == SM_RC_NOT_INVOCAB )
  {
    rc = SmGetVocWords ( reply, & num_missing, & missing );

    CheckSmRC("SmGetVocWords");

    if ( num_missing )
    {
#ifdef MISSINGVOCABS
      sprintf ( buffer, "Missing %d word(s) from '%s': ",
                num_missing, vocab_name );

      for ( i = 0 ; i < ( int ) num_missing ; i++ )
      {
        strcat ( buffer, missing [ i ].spelling );

        strcat ( buffer, " " );
      }

      fprintf (stderr, buffer );
#endif
    }

    return ( -1 );
  }

  /*--------------------------------------------------------------------*/
  /* Get the vocabulary name out of the reply structure so we will know */
  /* what grammar (vocabulary) to enable                                */
  /*--------------------------------------------------------------------*/
  rc = SmGetVocabName ( reply, & vocab_name );

  CheckSmRC("SmGetVocabName");

#ifdef MISSINGVOCABS
  sprintf ( buffer, "DefineGrammarCB: Defined '%s'", vocab_name );
  fprintf (stderr, buffer );
#endif

  rc = SmEnableVocab ( vocab_name, SmAsynchronous );

  CheckSmRC("SmGetEnableVocab");

  return ( 0 );
}

/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler GetNextWordCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("GetNextWordCB");

  /*-------------------------------------------------------------------*/
  /* This gets called whenever SmRecognizeNextWord() is called         */
  /*-------------------------------------------------------------------*/

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler RecoWordCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  int             rc;
  int             i;
  unsigned long   num_firm;
  SM_WORD       * firm;

  CheckSmRC("NextWordCB");

  /*-------------------------------------------------------------------*/
  /* SOMETIMES, the reco engine will have no idea what was said, and   */
  /* will call the recognized word callback - an application had best  */
  /* have a RecognizedWord callback setup to tell the engine to go     */
  /* again (SmRecognizeNextWord) if this happens..                     */
  /*-------------------------------------------------------------------*/

  rc = SmGetFirmWords ( reply, & num_firm, & firm );

#ifdef RECOMSGS
  for ( i = 0 ; i < ( int ) num_firm; i++ )
  {
    sprintf ( buffer, "RecoWordCB: firm[%d] = '%s' ('%s')\n",
              i, firm [ i ].spelling, firm [ i ].vocab );
    fprintf(stdout, buffer);
  }
#endif

  /*-------------------------------------------------------------------*/
  /* Tell the recognizer to 'go' again.  It stops so that if we wanted */
  /* to, we could change vocabs...                                     */
  /*-------------------------------------------------------------------*/
  rc = SmRecognizeNextWord ( SmAsynchronous );

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler RecoPhraseCB ( SM_MSG reply, caddr_t client, caddr_t caller )
{
  MyClientData *myData = (MyClientData *) client;
  int       rc;
  SM_WORD * firm;
  unsigned  long num_firm;
  int       i;
  char      buffer [ 255 ];
  char      phrase [ 255 ];
  char      *phraseArray[50];
  char      *translateTxt;
  char      phraseSend[512];
  unsigned  long flags;
  int       increment = 10;
  short     x, y;
  SM_ANNOTATION * annots;
  unsigned  long num_annots;

  SmGetRc ( reply, & rc );
  phraseSend[0] = '\0';
  /*-------------------------------------------------------------------*/
  /* Get the phrase that was recognized out of the reply structure.    */
  /*-------------------------------------------------------------------*/
  if ( rc != SM_RC_OK )
  {
    CheckSmRC ("RecoPhraseCB: SmGetRc returned not OK");

    rc = SmRecognizeNextWord ( SmAsynchronous );

    return ( rc );
  }


  /*------------------------------------------------------------------*/
  /* Get the 'phrase state' - this tells (among other things) whether */
  /* the phrase was accepted or rejected by the engine                */
  /*------------------------------------------------------------------*/
  rc = SmGetPhraseState ( reply, & flags );

  if ( rc != SM_RC_OK )
  {
    CheckSmRC ("RecoPhraseCB: SmGetPhraseState returned not OK ");

    rc = SmRecognizeNextWord ( SmAsynchronous );

    return ( rc );
  }

  /*------------------------------------------------------------------*/
  /* As with a recognized command, extract the recognized words       */
  /* (the 'phrase') from the reply structure                          */
  /*------------------------------------------------------------------*/
  rc = SmGetFirmWords ( reply, & num_firm, & firm );

  if ( rc != SM_RC_OK )
  {
    CheckSmRC ("RecoPhraseCB: SmGetFirmWords returned not OK");

    rc = SmRecognizeNextWord ( SmAsynchronous );

    return ( rc );
  }


  /*------------------------------------------------------------------*/
  /* Check the phrase state to see if it was accepted or not..        */
  /*------------------------------------------------------------------*/
  if ( flags & SM_PHRASE_ACCEPTED)   //   || flags & SM_PHRASE_HALTED)
  {
#ifdef RECOMSGS
    strcpy ( phrase, "Acc: " );
#endif
    if (num_firm > 50 )
    {
#ifdef RECOMSGS
      strcpy (phrase, "Too big to be translated");
#endif
    }
    else
    {
      
      strcat (phraseSend, "       " );
      for ( i = 0 ; i < num_firm ; i++ )
      {
        phraseArray[i] = firm[i].spelling;
        strcat (phraseSend, firm [ i ].spelling );
        strcat (phraseSend, " " );
      }
      phraseArray[i] = NULL;
      VtGetTranslation (fsg, phraseArray, &translateTxt);
      DisplaySendMessage (displayLabel, phraseSend, child, parent, root, False);
      DisplaySendMessage (displayLabel, translateTxt, child, parent, root, CalcFound);
    }
  }
  else
  {
#ifdef RECOMSGS
    strcpy ( phrase, "Phrase Word Reco: " );
#endif

  }


  /*------------------------------------------------------------------*/
  /* And go through them, making a string that has the complete       */
  /* phrase in it so that it can be displayed                         */
  /*------------------------------------------------------------------*/
  for ( i = 0 ; i < num_firm ; i++ )
  {
    strcat ( phrase, firm [ i ].spelling );

    strcat ( phrase, " " );

  }




#ifdef RECOMSGS
  fprintf (stderr,  phrase );
#endif

  /*------------------------------------------------------------------*/
  /* If the phrase was accepted, then move the toplevel window a bit  */
  /* in whatever direction was specified (the direction is determined */
  /* by looking at the 1st character of the 3rd word of the phrase,   */
  /* the amount is determined by the annotation on the last word of   */
  /* the phrase (if it exists; if it doesn't, the default us used)    */
  /*------------------------------------------------------------------*/
  if ( flags & SM_PHRASE_ACCEPTED )
  {
    /*----------------------------------------------------------------*/
    /* Get the annotation that's tied to the last part of the phrase  */
    /* (it contains the amount we want to move the window)..  If it's */
    /* not there or is the wrong type, then the grammar has probably  */
    /* changed since this part of the code was written..              */
    /*----------------------------------------------------------------*/
    rc = SmGetAnnotations ( reply, & num_annots, & annots );


  }


  /*------------------------------------------------------------------*/
  /* Tell the engine to 'go' again                                    */
  /*------------------------------------------------------------------*/
  rc = SmRecognizeNextWord ( SmAsynchronous );

  return ( rc );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler UtteranceCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  /*************
  LogMessage ( "UtteranceCB\n" );
  ************/
  /*-------------------------------------------------------------------*/
  /* The engine has turned the mic off and processed all of the audio  */
  /*-------------------------------------------------------------------*/

  return ( SM_RC_OK );
  /*-------------------------------------------------------------------*/
  /* The engine has turned the mic off and processed all of the audio  */
  /*-------------------------------------------------------------------*/

  return ( SM_RC_OK );
}





void DisplaySendMessage (Widget displayLabel, char *msg, Window child, Window parent, Window root, Boolean sendFlag)
{
  XEvent     generatedKeyEvent;
  XEvent     destroyEvent;
  int k, i;
  KeySym digitKeySym;
  int x, y;
  XmString xmstrLabel;
  char calChar;


 
  XtUnmapWidget (displayLabel);


  if (sendFlag) 
  {
     k = strlen(msg);

#ifdef DEBUGMSGS         
     printf ("String form of the value is %s\n", msg);
#endif
     i = 0;
     while (k-- > 0)
     {

            calChar = msg[i];
            if (calChar == 32)
            {
               i++;
               continue;
            }
            else if (calChar == 'm')
            {
                digitKeySym = XK_KP_Multiply;
            }
            else if (calChar == 'b')
            {
                digitKeySym = 32; 
            }
            else
            {
                digitKeySym = (unsigned int) calChar;
            }
            XTranslateCoordinates (dspG, child, root, 0, 0, &x, &y, &parent);
            generatedKeyEvent.xkey.type = KeyPress;
            generatedKeyEvent.xkey.serial = 1234;    
            generatedKeyEvent.xkey.send_event = False;
            generatedKeyEvent.xkey.display = dspG;
            generatedKeyEvent.xkey.window = child;
            generatedKeyEvent.xkey.subwindow = None;        
            generatedKeyEvent.xkey.time = CurrentTime;
            generatedKeyEvent.xkey.root = root;                               
            generatedKeyEvent.xkey.x_root = x;         
            generatedKeyEvent.xkey.y_root = y;         
            generatedKeyEvent.xkey.x = 0;         
            generatedKeyEvent.xkey.y = 0;         
            generatedKeyEvent.xkey.state = 0;           
            generatedKeyEvent.xkey.same_screen = True;
            generatedKeyEvent.xkey.keycode = XKeysymToKeycode (dspG, digitKeySym);
#ifdef DEBUGMSGS  
            printf ("Ascii value of send char is %x\n", digitKeySym);
#endif
            XSendEvent (dspG, child, True, KeyPressMask, &generatedKeyEvent);
            i++;
     }
  }
  else
  {
     xmstrLabel = XmStringCreateSimple (msg);

     XtVaSetValues (displayLabel, XmNlabelString, xmstrLabel, NULL);

     XmStringFree (xmstrLabel);

   }

   XtMapWidget (displayLabel);

   if (calChar == 'q')
   {
      xtIntrvlIdForClose = XtAppAddTimeOut (theAppG, CLOSE_TIME,  CloseControlCB, popupShellG);
   }

}

Boolean GetCalcWindow ( Display * display, Window the_root,  Window *child, Window *parent)
{
  Window     root, tempparent, *children;
  Window     the_child;
  char     * the_name;
  int        num_kids;
  int        i, n;
  Arg        args[5];
  int win_count;
  char *name_string;

  XQueryTree ( display, the_root, &root, &tempparent, &children, &num_kids );

  for ( i = 0 ; i < num_kids ; i++ )
  {
    if ( children[i] == popup_window )
    {
      continue;
    }

    if (GetWindowInfoRecursive ( display, children[i], &the_child) )
    {
       *child = the_child;
       *parent = children[i];
       return (True); 
    }
  }
  return (False);

}

Boolean GetWindowInfoRecursive (  Display * display, Window child,  Window  * the_child  )
{
  static     Boolean Found = False;   
  Window     root, parent, *children;
  Window     temp;
  int name_rc;
  unsigned int num_kids;
  char *name_string;
  int returnVal;

  if (XQueryTree ( display, child, &root, &parent, &children, &num_kids ))
  {
     while (num_kids--)
     {
        GetWindowInfoRecursive (display, children[num_kids], the_child);
        name_rc = XFetchName (display, children[num_kids], &name_string );
        if (name_rc)
        {
            if (!strcmp (name_string, "Calculator"))
            {
               *the_child = children[num_kids];
               Found = True;
            }
        }
     }
  }
  return (Found);

}



static void CloseControlCB (Widget w, XtPointer client_data, XtPointer call_data)
{
   if (xtIntrvlIdForClose)
   {
     XtRemoveTimeOut (xtIntrvlIdForClose);
   }
   SmClose();
   XtDestroyWidget (XtParent (w));
   XtDestroyApplicationContext (theAppG);
   exit(0);
}

main (int argc, char **argv)
{
   char *argument;
   int decor; 
   int micLevel;
   Atom delMainWin;
   Widget shell,frame,form,form1;
   Widget popupShell;
   Atom delPopupShellWin;
   Pixel fg, bg;
   Widget frame1;


   int n;
   MyClientData  MyData;
   Arg       args[10];
   unsigned long ulDecor;

    if (argc > 1)
    {
       argument = argv[1];
       if (argument[0] == '-'  && argument[1] == 'r')
       {
          exit(0);
       }
       else
       {
          CalCtrlUsage();
          exit(0);
       }
     }
           
    
    // Initialize the Intrinsics
    
    XtToolkitInitialize ();
    
    // Create an application context
    
    theAppG = XtCreateApplicationContext ();
    
    // Open a connection to the X server
    
    dspG = XtOpenDisplay (theAppG, NULL, NULL, 
			"CalCtrl", 
			NULL, 0, 
			(Cardinal *) &argc, argv);
     

    // Create a toplevel shell

    mainTopShellG = XtVaAppCreateShell ("calctrl", "CalCtrl",
				applicationShellWidgetClass,
				dspG, 
                                NULL);
  


    n = 0;
    popupShell = XtCreatePopupShell ( "PopupShell", topLevelShellWidgetClass, 
                               mainTopShellG, args, n );

    popupShellG = popupShell;
    XtVaGetValues (popupShell, XmNmwmDecorations, &decor, NULL);
    ulDecor &= ~MWM_DECOR_TITLE;
    ulDecor &= ~MWM_DECOR_RESIZEH;
    ulDecor &= ~MWM_DECOR_MINIMIZE;
    ulDecor &= ~MWM_DECOR_MAXIMIZE;
    XtVaSetValues (popupShell, XmNmwmDecorations, ulDecor, NULL);

    MyData.top_level = mainTopShellG;

    frame = ( Widget ) XtVaCreateWidget ("TopForm", xmFrameWidgetClass,
                                         popupShell,
                                         XmNwidth, DisplayWidth(dspG, 0)*99/100,
                                         XmNshadowType, XmSHADOW_ETCHED_OUT, 
                                         XmNshadowThickness, 3,
                                         NULL);

    form = ( Widget ) XtVaCreateWidget ("TopForm", xmFormWidgetClass,
                                         frame,
                                         XmNwidth, DisplayWidth (dspG, 0)*99/100,
                                         XmNfractionBase,12,
                                         NULL);
     
    

    XtVaGetValues (form,  XmNforeground, &fg, XmNbackground, &bg, NULL);
    btnPixmapOff = XmGetPixmap (XtScreen(form), "./enmic2.xpm", fg, bg);
    btnPixmapOn = XmGetPixmap (XtScreen(form), "./enmic3.xpm", fg, bg);
    micButton = ( Widget ) XtVaCreateWidget ("MicButton",
                                         xmPushButtonWidgetClass,
                                         form,
                                         XmNtopAttachment,   XmATTACH_POSITION,
                                         XmNtopPosition,      0,
				         XmNleftAttachment,  XmATTACH_POSITION,
				         XmNleftPosition,     0,                
                                         XmNbottomAttachment, XmATTACH_POSITION,
                                         XmNbottomPosition, 12,
                                         XmNlabelType, XmPIXMAP,
                                         XmNlabelPixmap, btnPixmapOff,
                                         NULL);

                                         
    XtAddCallback ( micButton, XmNactivateCallback, ToggleMic, NULL );


    frame1 = ( Widget ) XtVaCreateWidget ("Frame1",
                                         xmFrameWidgetClass,
                                         form,
                                         XmNtopAttachment,   XmATTACH_POSITION,
                                         XmNtopPosition,      0,
                                         XmNrightAttachment,   XmATTACH_POSITION,
                                         XmNrightPosition,      12,
				         XmNbottomAttachment,  XmATTACH_POSITION,
				         XmNbottomPosition,     12,                
                                         XmNshadowType, XmSHADOW_IN,//ETCHED_IN,
                                         XmNshadowThickness, 3,
                                         NULL);


    form1 = ( Widget ) XtVaCreateWidget ("Form1",
                                         xmFormWidgetClass,
                                         frame1,
                                         XmNfractionBase,  12,
                                         NULL);

    userLabel = ( Widget ) XtVaCreateWidget ("UserLabel",
                                         xmLabelWidgetClass,
                                         form1,
                                         XmNtopAttachment,   XmATTACH_POSITION,
                                         XmNtopPosition,      0,
				         XmNrightAttachment,  XmATTACH_POSITION,
				         XmNrightPosition,  12,                 
                                         XmNrightOffset, 5,
                                         XmNbottomAttachment, XmATTACH_POSITION,
                                         XmNbottomPosition, 12,
                                         XmNshadowType, XmSHADOW_IN, 
                                         XmNshadowThickness, 3,
                                         NULL);


    displayLabel = XtVaCreateWidget ("DisplayLabel",
                                         xmLabelWidgetClass, form, 
                                         XmNalignment, XmALIGNMENT_BEGINNING,
                                         XmNtopAttachment,   XmATTACH_POSITION,
                                         XmNtopPosition,      0,
				         XmNleftAttachment,  XmATTACH_WIDGET,
				         XmNleftWidget,     micButton, 
				         XmNrightAttachment,  XmATTACH_WIDGET,
				         XmNrightWidget, frame1,                 
                                         XmNbottomAttachment, XmATTACH_POSITION,
                                         XmNbottomPosition, 12,
                                         NULL);          


    popup_window = XtWindow ( popupShell );
    XtVaSetValues (popupShell,
                     XmNx,  DisplayWidth (dspG, 0)*1/200,
                     XmNy, 0,
                     NULL);
   


    delPopupShellWin = XmInternAtom (dspG, "WM_DELETE_WINDOW", False);

    XmAddWMProtocolCallback (popupShell, delPopupShellWin, CloseControlCB, NULL);

    XtManageChild ( micButton );
    XtManageChild ( displayLabel );
    XtManageChild ( frame1 );
    XtManageChild ( form1 );
    XtManageChild ( userLabel );
    XtManageChild ( form );
    XtManageChild ( frame );
    XtPopup (popupShell, XtGrabNone); 


    root = DefaultRootWindow ( dspG );

    if (GetCalcWindow (dspG, root,  &child, &parent))
    {
        DisplaySendMessage (displayLabel, labelString, child, parent, root, False);
        CalcFound = True;
    }

   //Realize all widgets and enter the main event loop

    ConnectStuff ( &MyData );
    
    XtAppMainLoop(theAppG);
    
}


void CalCtrlUsage(void)
{
    fprintf(stdout,"  Usage:\n");
    fprintf(stdout,"  CalCtrl \n");
    fprintf(stdout,"     No option : Display GUI \n");
}
