#///////////////////////////////////////////////////////////////////////////////
#//===========================================================================//
#// COPYRIGHT:                                                                //
#// ----------                                                                //
#// Copyright (C) International Business Machines Corp., 1999.                //
#//                                                                           //
#// DISCLAIMER OF WARRANTIES:                                                 //
#// -------------------------                                                 //
#// The following [enclosed] code is sample code created by IBM               //
#// Corporation.  This sample code is not part of any standard IBM            //
#// product and is provided to you solely for the purpose of                  //
#// assisting you in the development of your applications.  The               //
#// code is provided "AS IS", without warranty of any kind.  IBM              //
#// shall not be liable for any damages arising out of your use of            //
#// the sample code, even if they have been advised of the                    //
#// possibility of such damages.                                              //
#//===========================================================================//
#//////////////////////////////////////////////////////////////////////////////
#//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>

#include <smapi.h>
#include <dmacros.h>

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif


#define CheckEndOfString(charcoal)   \
        if ( *charcoal == '\0' )     \
        {                            \
          return ( charcoal );       \
        }                            \


static DmMacro   macros [ 250 ];
static int       num_macros = 0;
static char    * names [ 250 ];


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static char * SkipWhiteSpace ( char * cp )
{
  while ( ( *cp != '\0' ) && ( ( *cp == ' ' ) || ( *cp == '\t' ) ) )
  {
    cp++;
  }

  return ( cp );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static int UpperCase ( char * cp )
{
  while ( * cp )
  {
    * cp = toupper ( * cp );
    cp = cp + 1;
  }

  return ( 0 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int UnescapeString ( char * cp )
{
  int  frompos;
  int  topos;
  char temp [ 1024 ];

  frompos = 0;
  topos = 0;

  while ( cp [ frompos ] )
  {
    if ( cp [ frompos ] != '\\' )
    {
      temp [ topos ] = cp [ frompos ];
      frompos = frompos + 1;
      topos = topos + 1;
    }
    else
    {
      frompos = frompos + 1;

      switch ( cp [ frompos ] )
      {
        case 'n': temp [ topos ] = '\n'; break;
        case 't': temp [ topos ] = '\t'; break;
        case '\'': temp [ topos ] = '\''; break;
      }

      frompos = frompos + 1;
      topos = topos + 1;
    }
  }

  temp [ topos ] = '\0';

  strcpy ( cp, temp );

  return ( topos );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static char * ReadNextLine ( FILE * fp )
{
  static char   line_buff [ 10240 ];
  char        * cp;
  int           len;

  /* First, read in the line */
  cp = fgets ( line_buff, sizeof ( line_buff ), fp );

  if ( cp == NULL )
  {
    return ( NULL );
  }

  len = strlen ( line_buff );

  /* If the last character is a newline, then toss it */
  if ( line_buff [ len - 1 ] == '\n' )
  {
    line_buff [ --len ] = '\0';
  }


  /* Now check to see if there's a backslash at the end.. */

  while ( line_buff [ len - 1 ] == '\\' )
  {
    line_buff [ --len ] = '\0';

    cp = & line_buff [ len ];

    cp = fgets ( cp, 1024, fp );

    if ( cp == NULL )
    {
      return ( line_buff );
    }

    len = len + strlen ( cp );

    if ( line_buff [ len - 1 ] == '\n' )
    {
      line_buff [ --len ] = '\0';
    }
  }

  return ( line_buff );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static char * GetTag ( char * cp )
{
  static char   tag_buff [ 80 ];
  char        * tp;

  memset ( tag_buff, '\0', sizeof ( tag_buff ) );

  tp = tag_buff;

  while ( ( *cp ) && ( *cp != ' ' ) && ( *cp != '\t' ) && ( *cp != '=' ) )
  {
    *tp = *cp;
    tp = tp + 1;
    cp = cp + 1;
  }

  *tp = '\0';

  return ( tag_buff );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static char * GetString ( char * cp, int * xtra )
{
  static char   str_buff [ 1024 ];
  char        * sp;
  short         quoted = FALSE;
  short         escaped = FALSE;
  char          last;

  /* Check to see if the first character is a quote.  If it is, then we */
  /* strip it off and remember that it was there                        */
  if ( *cp == '\'' )
  {
    quoted = TRUE;
    cp++;
  }

  * xtra = 0;

  sp = str_buff;

  /* if the string was quoted, then we copy everything we find until   */
  /* we find the end quote (and then we toss that).  Otherwise, we     */
  /* stop when we hit whitespace                                       */
  if ( quoted )
  {
    /* Gotta handle an escaped quote in here!!! */
    while ( ( *cp != '\0' ) && ( ( *cp != '\'' ) || ( escaped ) ) )
    {
      escaped = FALSE;

      if ( *cp == '\\' ) escaped = TRUE;

      *sp = *cp;
      sp++;
      cp++;
    }

    if ( *cp == '\'' )
    {
      * xtra = 2;
    }
  }
  else
  {
    while ( ( *cp != '\0' ) && ( *cp != ' ' ) )
    {
      *sp = *cp;
      sp++;
      cp++;
    }
  }

  *sp = '\0';

  return ( str_buff );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static char * GetNextPair ( char * lp, char * * tag, char * * val )
{
  static char   tag_buff [ 255 ];
  static char   val_buff [ 1024 ];
  int           xtra;
  int           len;
  char        * cp;
  char        * temp;

  tag_buff [ 0 ] = '\0';
  val_buff [ 0 ] = '\0';

  cp = SkipWhiteSpace ( lp );

  temp = GetTag ( cp );

  len = strlen ( temp );

  strncpy ( tag_buff, temp, len );
  tag_buff [ len ] = '\0';

  * tag = tag_buff;

  cp = cp + len;  /* cp now points to whatever ended the tag */

  CheckEndOfString(cp);

  /* Skip over white space that might have ended the search */

  cp = SkipWhiteSpace ( cp );

  CheckEndOfString(cp);

  /* cp now points to the equal sign.  Skip it, and then skip over */
  /* the white space that might be following it                    */
  cp = cp + 1;
  CheckEndOfString(cp);

  cp = SkipWhiteSpace ( cp );

  CheckEndOfString(cp);

  /* now we're pointing at the start of the data that goes with this */
  /* tag (a quoted or non-quoted string)                             */

  temp = GetString ( cp, & xtra );

  len = strlen ( temp );

  strncpy ( val_buff, temp, len );
  val_buff [ len ] = '\0';

  * val = val_buff;

  cp = cp + len + xtra;

  return ( cp );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static int ParseFlags ( char * cp )
{
  char * start;
  int    len;
  int    flags;
  char   temp [ 255 ];

  /* check for end of string */

  flags = 0;

  while ( * cp != 0 )
  {
    cp = SkipWhiteSpace ( cp );

    start = cp;

    /* Read until a space or a plus sign */
    while ( ( * cp ) && ( * cp != ' ' ) && ( * cp != '+' ) )
    {
      cp = cp + 1;
    }

    len = cp - start;

    /* End of the string (space or plus sign) was found, now go and see */
    /* what the flag is                                                 */
    if ( strncmp ( start, "NOTHING", len ) == 0 )
    {
    }

    if ( strncmp ( start, "CAPITALIZE", len ) == 0 )
    {
      flags = flags | SM_WORD_CAPITALIZE;
    }

    if ( ( strncmp ( start, "JOIN-PREVIOUS", len ) == 0 ) ||
         ( strncmp ( start, "JOIN_PREVIOUS", len ) == 0 ) )
    {
      flags = flags | SM_WORD_JOINPREVIOUS;
    }

    if ( ( strncmp ( start, "JOIN-NEXT", len ) == 0 ) ||
         ( strncmp ( start, "JOIN_NEXT", len ) == 0 ) )
    {
      flags = flags | SM_WORD_JOINNEXT;
    }

    if ( strncmp ( start, "DIGIT", len ) == 0 )
    {
      flags = flags | 0x08;
    }

    /* If it was whitespace that ended the search above, then we now skip */
    /* over whatever whitespace is there                                  */
    cp = SkipWhiteSpace ( cp );

    /* Or, if the search was ended by a plus sign (or we're at one now),  */
    /* then skip over it too..                                            */
    if ( ( * cp ) && ( * cp == '+' ) ) cp = cp + 1;
  }

  return ( flags );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static int ParseLine ( char * lp, DmMacro * mp )
{
  char * cp;
  char * tag;
  char * val;
  int    len;

  mp -> flags = 0;
  mp -> name [ 0 ] = '\0';
  mp -> text [ 0 ] = '\0';

  cp = lp;

  while ( ( cp = GetNextPair ( cp, & tag, & val ) ) != NULL )
  {
    UpperCase ( tag );

    if ( strcmp ( tag, "NAME" ) == 0 )
    {
      strcpy ( mp->name, val );

      continue;
    }

    if ( strcmp ( tag, "TEXT" ) == 0 )
    {
      UnescapeString ( val );

      strcpy ( mp->text, val );

      continue;
    }

    if ( strcmp ( tag, "FLAGS" ) == 0 )
    {
      mp -> flags = ParseFlags ( val );
    }

    if ( ! *cp )
    {
      break;
    }
  }
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
static int LoadMacros ( char * fn )
{
  char * cp;
  FILE * fp;

  num_macros = 0;

  fp = fopen ( fn, "r" );

  if ( fp == NULL )
  {
    printf ( "Failed to open %s\n", fn );
    return ( -1 );
  }

  while ( ( cp = ReadNextLine ( fp ) ) != NULL )
  {
    cp = SkipWhiteSpace ( cp );

    if ( * cp != '\0' )
    {
      ParseLine ( cp, & macros [ num_macros ] );
      names [ num_macros ] = macros [ num_macros ].name;
      num_macros++;
    }
  }

  return ( num_macros );
}

/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
DmMacro * DmQueryMacroName ( char * name )
{
  int i;

  for ( i = 0 ; i < num_macros; i++ )
  {
    if ( strcmp ( name, macros [ i ].name ) == 0 )
    {
      return ( & macros [ i ] );
    }
  }

  return ( NULL );
}



/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int DmGetMacroNames ( char * * * the_names, int * count )
{
  * the_names = names;
  * count = num_macros;

  return ( num_macros );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int DmLoadMacros ( char * fn )
{
  int rc;

  rc = LoadMacros ( fn );

  printf ( "LoadMacros rc = %d\n", rc );
}
