///////////////////////////////////////////////////////////////////////////////
//===========================================================================//
// COPYRIGHT:                                                                //
// ----------                                                                //
// Copyright (C) International Business Machines Corp., 1999.                //
//                                                                           //
// DISCLAIMER OF WARRANTIES:                                                 //
// -------------------------                                                 //
// The following [enclosed] code is sample code created by IBM               //
// Corporation.  This sample code is not part of any standard IBM            //
// product and is provided to you solely for the purpose of                  //
// assisting you in the development of your applications.  The               //
// code is provided "AS IS", without warranty of any kind.  IBM              //
// shall not be liable for any damages arising out of your use of            //
// the sample code, even if they have been advised of the                    //
// possibility of such damages.                                              //
//===========================================================================//
//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>

#include <smapi.h>

#include <dmacros.h>

#include <dw.h>

#include <guifns.h>

/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/

static void ConnectStuff ( );


static unsigned long los_tagos [ 1000 ];
static int           num_tagos = 0;

static char          buffer [ 255 ];


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
void LogMessage ( char * buff )
{
  printf ( "%s\n", buff );
}


/*------------------------------------------------------------------------*/
/*                                                                        */
/*------------------------------------------------------------------------*/
int main ( int argc, char * * argv )
{
  GuiCreate ( argc, argv );

  ConnectStuff ( );

  GuiMainLoop ( );
  
  return ( 0 );       
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void ConnectStuff ( )
{
  static int first = TRUE;
  int        rc;
  int        smc;
  SmArg      smargs [ 30 ];
  char       * cp;
  static int input_id;

  SmHandler ConnectCB     ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler DisconnectCB  ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler SetCB         ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler MicOnCB       ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler MicOffCB      ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler DefineVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler EnableVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler GetNextWordCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler RecoWordCB    ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler RecoTextCB    ( SM_MSG reply, caddr_t client, caddr_t call_data );
  SmHandler UtteranceCB   ( SM_MSG reply, caddr_t client, caddr_t call_data );

  LogMessage ( "ConnectStuff invoked" );

  if ( first )
  {
    smc = 0;
    SmSetArg ( smargs [ smc ], SmNapplicationName,  GuiAppName );  smc++;
    SmSetArg ( smargs [ smc ], SmNexternalNotifier, GuiNotifier ); smc++;

    rc = SmOpen ( smc, smargs );

    if ( rc != SM_RC_OK )
    {
      sprintf ( buffer, "SmOpen() failed, rc = %d", rc );

      LogMessage ( buffer );

      return;
    }

    SmAddCallback ( SmNconnectCallback,             ConnectCB,       NULL );
    SmAddCallback ( SmNdisconnectCallback,          DisconnectCB,    NULL );
    SmAddCallback ( SmNsetCallback,                 SetCB,           NULL );
    SmAddCallback ( SmNmicOnCallback,               MicOnCB,         NULL );
    SmAddCallback ( SmNmicOffCallback,              MicOffCB,        NULL );
    SmAddCallback ( SmNenableVocabCallback,         EnableVocabCB,   NULL );
    SmAddCallback ( SmNdefineVocabCallback,         DefineVocabCB,   NULL );
    SmAddCallback ( SmNrecognizeNextWordCallback,   GetNextWordCB,   NULL );
    SmAddCallback ( SmNrecognizedWordCallback,      RecoWordCB,      NULL );
    SmAddCallback ( SmNrecognizedTextCallback,      RecoTextCB,      NULL );
    SmAddCallback ( SmNutteranceCompletedCallback,  UtteranceCB,     NULL );

    first = FALSE;
  }

  smc = 0;
  SmSetArg ( smargs [ smc ], SmNuserId,       SM_USE_CURRENT );  smc++;
  SmSetArg ( smargs [ smc ], SmNenrollId,     SM_USE_CURRENT );  smc++;
  SmSetArg ( smargs [ smc ], SmNtask,         SM_USE_CURRENT );  smc++;
  SmSetArg ( smargs [ smc ], SmNrecognize,    TRUE           );  smc++;
  SmSetArg ( smargs [ smc ], SmNoverrideLock, TRUE           );  smc++;

  rc = SmConnect ( smc, smargs, SmAsynchronous );

  sprintf ( buffer, "ConnectStuff: SmConnect() rc = %d", rc );

  LogMessage ( buffer );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
void CloseUpShop ( )
{
  GuiDestroy ( );

  exit ( 0 );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
void PlayTags ( )
{
  SM_MSG  reply;
  int     rc;
  int     howto = SM_PLAY_WORDS_CONTIGUOUS;

  LogMessage ( "PlayTags called" );

  rc = SmPlayWords ( num_tagos, los_tagos, howto, SmAsynchronous );

  sprintf ( buffer, "PlayTags: SmPlayWords() rc = %d", rc );

  LogMessage ( buffer );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
void TurnMicOn ( )
{
  SM_MSG  reply;
  int     rc;

  LogMessage ( "TurnMicOn called" );

  rc = SmMicOn ( SmAsynchronous );

  sprintf ( buffer, "TurnMicOn: SmMicOn() rc = %d", rc );

  LogMessage ( buffer );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
void TurnMicOff ( )
{
  SM_MSG  reply;
  int     rc;

  LogMessage ( "TurnMicOff called" );

  rc = SmMicOff( SmAsynchronous );

  sprintf ( buffer, "TurnMicOff: SmMicOff() rc = %d", rc );

  LogMessage ( buffer );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void DoSomethingSimple ( )
{
  printf ( "DoSomethingSimple invoked\n" );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
void LoadDictationMacros ( )
{
  int          rc;
  int          i;
  SM_VOCWORD   voc_words [ 250 ];
  SM_VOCWORD * voc_ptrs [ 250 ];
  char       * * names;
  int          count;

  rc = DmLoadMacros ( "dmacros.txt" );

  if ( rc <= 0 )
  {
    printf ( "No macros?\n" );
    return;
  }

  rc = DmGetMacroNames ( & names, & count );

  for ( i = 0 ; i < count; i++ )
  {
    voc_ptrs [ i ] = & ( voc_words [ i ] );

    voc_words [ i ].spelling      = names [ i ];
    voc_words [ i ].spelling_size = strlen ( names [ i ] ) ;
    voc_words [ i ].flags         = 0;
  }

  rc = SmAddToVocab ( "text", count, voc_ptrs, SmAsynchronous );

  printf ( "SmAddToVocab rc = %d\n", rc );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler ConnectCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  int    rc;
  char * cp;

  printf ( "ConnectCB\n" );

  CheckSmRC("ConnectCB");

  LoadDictationMacros ( );

  SmEnableVocab ( "text", SmAsynchronous );

  SmSet ( SM_SAVE_AUDIO, TRUE, SmAsynchronous );

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler DisconnectCB  ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler SetCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler MicOnCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("MicOnCB");

  SmRecognizeNextWord ( SmAsynchronous );

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler MicOffCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("MicOffCB");

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler EnableVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("EnableVocabCB");

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler DefineVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  char * vocab;
  int    rc;

  CheckSmRC("DefineVocabCB");

  SmGetVocabName ( reply, & vocab );

  sprintf ( buffer, "DefineVocabCB: vocab = %s", vocab );

  LogMessage ( buffer );

  SmEnableVocab ( vocab, SmAsynchronous );

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler GetNextWordCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  CheckSmRC("GetNextWordCB");

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler RecoWordCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  int             rc;
  int             i;
  unsigned long   num_firm;
  SM_WORD       * firm;

  CheckSmRC("NextWordCB");

  rc = SmGetFirmWords ( reply, & num_firm, & firm );

  for ( i = 0 ; i < ( int ) num_firm; i++ )
  {
    sprintf ( buffer, "RecoWordCB: firm[%d] = '%s' ('%s')\n",
              i, firm [ i ].spelling, firm [ i ].vocab );
    LogMessage ( buffer );

    if ( strcmp ( firm [ i ].spelling, "hello" ) == 0 )
    {
      DoSomethingSimple ( );
    }
  }

  rc = SmRecognizeNextWord ( SmAsynchronous );

  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static char * MakeFormattedWord ( SM_WORD * firmword )
{
  static char    text_buff [ 255 ];
  static short   cap_next = TRUE;
  static short   join_to_prev = TRUE;
  static short   join_with_next = FALSE;
  static short   flag_type;
  static short   flag_valu;
  DmMacro      * le_macro;
  char         * le_text;

  text_buff [ 0 ] = '\0';

  le_macro = DmQueryMacroName ( firmword -> spelling );

  if ( le_macro )
  {
    // printf ( "Found a dictation macro for '%s'\n", firmword->spelling );
    le_text = le_macro->text;
  }
  else
  {
    le_text = firmword->spelling;
  }

  //---------------------------------------------------------------------
  // Check to see if the previous word had the 'capitalize next' flag set
  // and if so, capitalize the first character of this (the next) word
  //---------------------------------------------------------------------
  if ( cap_next )
  {
    le_text [ 0 ] = toupper ( le_text [ 0 ] );

    cap_next = FALSE;
  }


  flag_type = firmword->flags & SM_WORD_FMT_TYPE_MASK;
  flag_valu = firmword->flags & SM_WORD_FMT_VALUE_MASK;

  if ( le_macro )
  {
    flag_valu = flag_valu | le_macro->flags;
  }

  //---------------------------------------------------------------
  // Zip through the flags (if there are any) and infer whatever we
  // can from them (capitalized, digit, etc)
  //---------------------------------------------------------------
  if ( flag_type == SM_WORD_FMT_TYPE_BASIC )
  {
    //---------------------------------------------------------------
    // If the capitalize flag is set, 'remember' it for the next word
    // that we run through here with..
    //---------------------------------------------------------------
    if ( flag_valu & SM_WORD_CAPITALIZE )
    {
      cap_next = TRUE;
    }

    //---------------------------------------------------------------
    // Check for 'join to previous' - if its set, then this word is
    // put into the text control without any spaces before it..
    //---------------------------------------------------------------
    if ( flag_valu & SM_WORD_JOINPREVIOUS )
    {
      join_to_prev = TRUE;
    }

    //---------------------------------------------------------------
    // Save this flag for the next word (if it's set when we go to
    // process the next word, then we set join_to_prev under the
    // covers)
    //---------------------------------------------------------------
    if ( flag_valu & SM_WORD_JOINNEXT )
    {
      join_with_next = TRUE;
    }

    if ( flag_valu & SM_WORD_DIGIT )
    {
    }
  }

  //---------------------------------------------------------------
  // If not join to previous, stick a blank in the text control
  //---------------------------------------------------------------
  if ( ! join_to_prev )
  {
    strcat ( text_buff, " " );
  }
  join_to_prev = FALSE;

  strcat ( text_buff, le_text );

  //---------------------------------------------------------------
  // If the join with next flag was set for this word (a dash does
  // this), then set the join to previous for the next word so
  // things get glued correctly..
  //---------------------------------------------------------------
  if ( join_with_next )
  {
    join_to_prev = TRUE;
  }
  join_with_next = FALSE;

  return ( text_buff );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler RecoTextCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  int             rc;
  int             i;
  unsigned long   num_firm;
  SM_WORD       * firm;
  unsigned long   num_infirm;
  SM_WORD       * infirm;
  char            text_buff [ 255 ];

  CheckSmRC("RecoTextCB");

  text_buff [ 0 ] = '\0';

  rc = SmGetFirmWords ( reply, & num_firm, & firm );

  for ( i = 0 ; i < ( int ) num_firm; i++ )
  {
    strcat ( text_buff, MakeFormattedWord ( & firm [ i ] ) );

    los_tagos [ num_tagos ++ ] = firm [ i ].tag;
  }

  GuiInsertText ( text_buff );

  rc = SmGetInfirmWords ( reply, & num_infirm, & infirm );

  for ( i = 0 ; i < ( int ) num_infirm; i++ )
  {
    sprintf ( buffer, "RecoTextCB: infirm[%d] = '%s' ('%s')\n",
              i, infirm [ i ].spelling, infirm [ i ].vocab );
    LogMessage ( buffer );
  }


  return ( SM_RC_OK );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
SmHandler UtteranceCB ( SM_MSG reply, caddr_t client, caddr_t call_data )
{
  LogMessage ( "UtteranceCB\n" );

  return ( SM_RC_OK );
}
