#///////////////////////////////////////////////////////////////////////////////
#//===========================================================================//
#// COPYRIGHT:                                                                //
#// ----------                                                                //
#// Copyright (C) International Business Machines Corp., 1999.                //
#//                                                                           //
#// DISCLAIMER OF WARRANTIES:                                                 //
#// -------------------------                                                 //
#// The following [enclosed] code is sample code created by IBM               //
#// Corporation.  This sample code is not part of any standard IBM            //
#// product and is provided to you solely for the purpose of                  //
#// assisting you in the development of your applications.  The               //
#// code is provided "AS IS", without warranty of any kind.  IBM              //
#// shall not be liable for any damages arising out of your use of            //
#// the sample code, even if they have been advised of the                    //
#// possibility of such damages.                                              //
#//===========================================================================//
#//////////////////////////////////////////////////////////////////////////////
#//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <gtk/gtk.h>

#include <dw.h>

#include <guifns.h>


/*------------------------------------------------------------------------*/
/*                                                                        */
/*------------------------------------------------------------------------*/
static void GuiExit      ( GtkWindow * btn, gpointer data );

static void GuiMicOn     ( GtkWindow * btn, gpointer data );

static void GuiMicOff    ( GtkWindow * btn, gpointer data );

static void GuiPlayTags  ( GtkWindow * btn, gpointer data );


static char buffer [ 255 ];

static GtkWidget * reco_text;

static GdkFont   * fixed_font;


/*------------------------------------------------------------------------*/
/*                                                                        */
/*------------------------------------------------------------------------*/
static GtkItemFactoryEntry menu_items[] =
{
  { "/_File",             NULL,         NULL,          0,  "<Branch>" },
  { "/File/Quit",         "<control>Q", GuiExit,       0,  NULL       },
  { "/_Speech",           NULL,         NULL,          0,  "<Branch>" },
  { "/Speech/Mic O_n",    NULL,         GuiMicOn,      0,  NULL       },
  { "/Speech/Mic O_ff",   NULL,         GuiMicOff,     0,  NULL       },
  { "/Speech/Play _Tags", NULL,         GuiPlayTags,   0,  NULL       },
  { "/_Help",             NULL,         NULL,          0,  "<LastBranch>" },
  { "/_Help/About",       NULL,         NULL,          0,  NULL       },
};


char * GuiAppName = "GtkDw";


/*------------------------------------------------------------------------*/
/*                                                                        */
/*------------------------------------------------------------------------*/
static void MakeMenus ( GtkWidget *window, GtkWidget ** menubar )
{
  int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new();

  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
				       accel_group);

  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);

  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));

  if (menubar)
  {
    *menubar = gtk_item_factory_get_widget(item_factory, "<main>");
  }
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
int GuiNotifier ( int socket_handle, int ( * recv_fn ) ( ),
                  void * recv_data, void * client_data )
{
  static int x_input_id = -1;

  if ( recv_fn == NULL )
  {
    gdk_input_remove ( x_input_id );

    return ( 0 );
  }

  x_input_id = gdk_input_add ( socket_handle, 1,
	                       ( GdkInputFunction ) recv_fn, recv_data );
  return ( 0 ) ;
}


/*------------------------------------------------------------------------*/
/*                                                                        */
/*------------------------------------------------------------------------*/
int GuiCreate ( int argc, char * * argv )
{
  GtkWidget * window;
  GtkWidget * vbox;
  GtkWidget * menubar;
  GtkWidget * table;
  GtkWidget * vscrollbar;
  GtkWidget * text;

  gtk_init ( & argc, & argv );
 
  window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );

  gtk_widget_set_usize ( window, 500, 400 );
  gtk_window_set_policy ( GTK_WINDOW ( window ), TRUE, TRUE, FALSE );  
  gtk_signal_connect ( GTK_OBJECT ( window ), "destroy",
                       GTK_SIGNAL_FUNC ( GuiExit ), NULL );

  gtk_window_set_title ( GTK_WINDOW ( window ), "Gtk Ventana de Dictado" );
  gtk_container_border_width ( GTK_CONTAINER ( window ), 0 );
  
  vbox = gtk_vbox_new ( FALSE, 1 );
  gtk_container_border_width ( GTK_CONTAINER ( vbox ), 1 );   
  gtk_container_add ( GTK_CONTAINER ( window ), vbox );       
  gtk_widget_show ( vbox );                                   

  MakeMenus ( window, & menubar );
  gtk_box_pack_start ( GTK_BOX ( vbox ), menubar, FALSE, TRUE, 0 );
  gtk_widget_show ( menubar );

  table = gtk_table_new ( 2, 2, FALSE );
  gtk_table_set_row_spacing ( GTK_TABLE ( table ), 0, 2 );
  gtk_table_set_col_spacing ( GTK_TABLE ( table ), 0, 2 );
  gtk_box_pack_start ( GTK_BOX ( vbox ), table, TRUE, TRUE, 0 );
  gtk_widget_show ( table );
  
  text = gtk_text_new ( NULL, NULL );

  reco_text = text;

  gtk_text_set_editable ( GTK_TEXT ( text ), TRUE );
  gtk_text_set_word_wrap ( GTK_TEXT ( text ), TRUE );

  gtk_table_attach (GTK_TABLE (table), text, 0, 1, 0, 1,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show ( text );

  vscrollbar = gtk_vscrollbar_new ( GTK_TEXT ( text )->vadj );
  gtk_table_attach ( GTK_TABLE ( table ), vscrollbar, 1, 2, 0, 1,
                     GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0 );
  gtk_widget_show ( vscrollbar );

  fixed_font = gdk_font_load ( "-adobe-courier-bold-r-normal--18-*" );

  gtk_widget_realize ( text );

  gtk_widget_show ( window );

  return ( 0 );       
}


/*------------------------------------------------------------------------*/
/*                                                                        */
/*------------------------------------------------------------------------*/
int GuiDestroy ( )
{
  gtk_main_quit ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
int GuiMainLoop ( )
{
  gtk_main ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiExit ( GtkWindow * btn, gpointer data )
{
  CloseUpShop ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiPlayTags ( GtkWindow * btn, gpointer data )
{
  PlayTags ( );
}


/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiMicOn ( GtkWindow * btn, gpointer data )
{
  TurnMicOn ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiMicOff ( GtkWindow * btn, gpointer data )
{
  TurnMicOff ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
int GuiInsertText ( char * text )
{
  gtk_text_freeze ( GTK_TEXT ( reco_text ) );

  gtk_text_insert ( GTK_TEXT ( reco_text ), fixed_font,
                    NULL, NULL, text, -1 );

  gtk_text_thaw ( GTK_TEXT ( reco_text ) );

  return ( 0 );
}
