#///////////////////////////////////////////////////////////////////////////////
#//===========================================================================//
#// COPYRIGHT:                                                                //
#// ----------                                                                //
#// Copyright (C) International Business Machines Corp., 1999.                //
#//                                                                           //
#// DISCLAIMER OF WARRANTIES:                                                 //
#// -------------------------                                                 //
#// The following [enclosed] code is sample code created by IBM               //
#// Corporation.  This sample code is not part of any standard IBM            //
#// product and is provided to you solely for the purpose of                  //
#// assisting you in the development of your applications.  The               //
#// code is provided "AS IS", without warranty of any kind.  IBM              //
#// shall not be liable for any damages arising out of your use of            //
#// the sample code, even if they have been advised of the                    //
#// possibility of such damages.                                              //
#//===========================================================================//
#//////////////////////////////////////////////////////////////////////////////
#//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>

#include <dw.h>

#include <guifns.h>


/*----------------------------------------------------*/
/* A couple of important values that get passed about */
/*----------------------------------------------------*/
typedef struct
{
  void * top_level;
  int    x_input_id;
} MyClientData;


/*---------------------------------------------------------------------*/
/* These are the callbacks for the different menu items...             */
/*---------------------------------------------------------------------*/
static void GuiExit     ( Widget w, XtPointer a, XtPointer b );

static void GuiMicOn    ( Widget w, XtPointer a, XtPointer b );

static void GuiMicOff   ( Widget w, XtPointer a, XtPointer b );

static void GuiPlayTags ( Widget w, XtPointer a, XtPointer b );

static void GuiSelect   ( Widget w, XtPointer a, XtPointer b );

/*---------------------------------------------------------------------*/
/* The 'helper' functions for selecting a window...                    */
/*---------------------------------------------------------------------*/
static Window GetWindowId ( Display * dpy );
static Window * GetWindowChildren ( Display * dpy, Window win, int * numKids );

/*---------------------------------------------------------------------*/
/* And a function to send key(s) to another window                     */
/*---------------------------------------------------------------------*/
void ProcessKey ( char ch );

void SendTheKey ( Display * display, Window window,
                  KeySym key, unsigned long shift_state, int type );


static Widget reco_text;

static char   buffer   [ 255 ];

static MyClientData myData;

char * GuiAppName = "TifDw";

static Display * target_dpy = NULL;
static Window    target_win = ( Window ) NULL;
static int       selected_win = FALSE;

/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
int GuiNotifier ( int socket_handle, int ( * recv_fn ) ( ),
                  void * recv_data, void * client_data )
{
  XtAppContext ctxt;

  if ( recv_fn == NULL )
  {
    XtRemoveInput ( myData.x_input_id );

    return ( 0 );
  }

  ctxt = XtWidgetToApplicationContext( myData.top_level );

  myData.x_input_id = XtAppAddInput ( ctxt, socket_handle,
                                      ( void * ) XtInputReadMask,
        ( void ( * ) ( void *, int *, unsigned long * ) ) recv_fn, recv_data );

  return ( 0 );
}


/*----------------------------------------------------------*/
/*                                                          */
/*----------------------------------------------------------*/
int GuiCreate ( int argc, char * * argv )
{
  Widget  top_level;
  Widget  main_window, menu_bar, cascade, pulldown, button, form;
  Arg     args [ 10 ];
  int     n;

  myData.top_level = XtInitialize ( "dwf", "Dwf", NULL, 0, & argc, argv );
  top_level = myData.top_level;

  n = 0;
  main_window = ( Widget ) XmCreateMainWindow ( top_level, "mainwindow",
                                                argc, n );

  n = 0 ;
  menu_bar = ( Widget ) XmCreateMenuBar ( main_window, "menubar", args, n );
  XtManageChild ( menu_bar );


  /*-------------------------------------------------------------------------*/
  /* GUI components for the File pulldown                                    */
  /*-------------------------------------------------------------------------*/
  pulldown = ( Widget ) XmCreatePulldownMenu ( menu_bar, "file_pulldown",
                                               args, 0 );

  button = ( Widget ) XmCreatePushButton ( pulldown, "select_button", args, 0 );
  XtManageChild ( button );
  XtAddCallback ( button, XmNactivateCallback, GuiSelect, NULL );

  button = ( Widget ) XmCreatePushButton ( pulldown, "exit_button", args, 0 );
  XtManageChild ( button );
  XtAddCallback ( button, XmNactivateCallback, GuiExit, NULL );

  n = 0;
  XtSetArg ( args [ n ], XmNsubMenuId, pulldown ); n++;
  cascade = ( Widget ) XmCreateCascadeButton ( menu_bar, "File", args, n );
  XtManageChild ( cascade );



  /*-------------------------------------------------------------------------*/
  /* GUI components for the Microphone                                       */
  /*-------------------------------------------------------------------------*/
  pulldown = ( Widget ) XmCreatePulldownMenu ( menu_bar, "mic_pulldown",
                                               args, 0 );
  button = ( Widget ) XmCreatePushButton ( pulldown, "micon_button",
                                           args, 0 );
  XtManageChild ( button );
  XtAddCallback ( button, XmNactivateCallback, GuiMicOn, NULL );

  button = ( Widget ) XmCreatePushButton ( pulldown, "micoff_button", args, 0 );
  XtManageChild ( button );
  XtAddCallback ( button, XmNactivateCallback, GuiMicOff, NULL );

  button = ( Widget ) XmCreatePushButton ( pulldown, "play_button", args, 0 );
  XtManageChild ( button );
  XtAddCallback ( button, XmNactivateCallback, GuiPlayTags, NULL );

  n = 0;
  XtSetArg ( args [ n ], XmNsubMenuId, pulldown ); n++;
  cascade = ( Widget ) XmCreateCascadeButton ( menu_bar, "Speech", args, n );
  XtManageChild ( cascade );


  n = 0;
  form = ( Widget ) XmCreateForm ( main_window, "work_area", args, n );
  XtManageChild ( form );

  n = 0;
  reco_text = ( Widget ) XmCreateScrolledText ( form, "reco_text", args, n );
  XtManageChild ( reco_text );

  XtManageChild ( main_window );

  XtRealizeWidget ( top_level );
}


/*----------------------------------------------------------*/
/*                                                          */
/*----------------------------------------------------------*/
int GuiMainLoop ( )
{
  XtMainLoop ( );
}


/*----------------------------------------------------------*/
/*                                                          */
/*----------------------------------------------------------*/
int GuiDestroy ( )
{
  return ( 0 );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiExit ( Widget w, XtPointer a, XtPointer b )
{
  CloseUpShop ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiPlayTags ( Widget w, XtPointer a, XtPointer b )
{
  PlayTags ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiMicOn ( Widget w, XtPointer a, XtPointer b )
{
  TurnMicOn ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiMicOff ( Widget w, XtPointer a, XtPointer b )
{
  TurnMicOff ( );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
int GuiInsertText ( char * text )
{
  char           * cp;
  XmTextPosition   pos;
  int              len;
  int              i;

  cp = ( char * ) XmTextGetString ( reco_text );

  pos = ( XmTextPosition ) strlen ( cp );

  XmTextInsert ( reco_text, pos, text );

  len = strlen ( text );

  XmTextSetInsertionPosition ( reco_text, pos + len );

  if ( selected_win )
  {
    for ( i = 0 ; i < len; i++ )
    {
      ProcessKey ( text [ i ] );
    }
  }

  return ( 0 );
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static void GuiSelect ( Widget widget, XtPointer data, XtPointer other )
{
  Window   root;
  Window   parent;
  Window * kids;
  int      numKids;
  Widget   topLevel;
  char   * which_win;
  char     buff [ 255 ];
  int      which_len;
  int      i;

  topLevel = myData.top_level;

  XmUpdateDisplay ( topLevel );

  XSync ( XtDisplay ( topLevel ), 0 );
  XSync ( XtDisplay ( topLevel ), 0 );
  XSync ( XtDisplay ( topLevel ), 0 );
  XSync ( XtDisplay ( topLevel ), 0 );

  target_dpy = XtDisplay ( topLevel );
  root = DefaultRootWindow ( target_dpy );
  target_win = GetWindowId ( target_dpy );

  printf ( "Selected window is %x\n", target_win );

  /*-------------------------------------------------------------------------*/
  /* Because it's anybody's guess how the window                             */
  /* manager may have reparented the windows, we need some help in finding   */
  /* the real target of our events.  The 'which_win' string says how many    */
  /* levels of reparenting have taken place, and which child at each level   */
  /* we should look at (the choices are '0' and 'n', for the first and the   */
  /* last).  The length of the string indicates the number of levels to      */
  /* traverse (if it's 2 characters long, the target is two levels down from */
  /* the top of the tree..                                                   */
  /*                                                                         */
  /* Hint: Under fvwm, it should be n0, and under Enlight*, 00 works         */
  /*-------------------------------------------------------------------------*/
  which_win = ( char * ) getenv ( "LEVEL_HACK" );
  if ( which_win )
  {
    printf ( "WhichWin is '%s'\n", which_win );
  }
  else
  {
    which_win = "00";
  }

  which_len = strlen ( which_win );

  for ( i = 0 ; i < which_len; i++ )
  {
    kids = GetWindowChildren ( target_dpy, target_win, & numKids );

    if ( ! kids )
    {
      printf ( "Ran out of kids at %d (is LEVEL_HACK set correctly?)\n", i );
      selected_win = FALSE;
      return;
    }

    printf ( "At %d, Choosing '%c' of %d\n", i, which_win [ i ], numKids );

    switch ( which_win [ i ] )
    {
      case '0':
           target_win = kids [ 0 ];
           break;

      case 'n':
      case 'N':
           target_win = kids [ numKids - 1 ];
           break;
    }

    XFree ( kids );
  }

  printf ( "Target is %x\n", target_win );

  selected_win = TRUE;

  return;
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static Window GetWindowId ( Display * dpy)
{
  Window  root;
  Cursor  hairs;
  XEvent  event;
  Window  new_root, child;
  int     mask, root_x, root_y, win_x, win_y;

  root = DefaultRootWindow ( dpy );
  hairs = XCreateFontCursor ( dpy, XC_crosshair );

  if ( XGrabPointer ( dpy, root, False, ButtonPressMask, GrabModeAsync,
                      GrabModeAsync, None, hairs, CurrentTime ) != GrabSuccess )
  {
    printf("Grab Error\n" );
    return ( ( Window ) NULL );
  }

  while ( 1 )
  {
    if ( XPending ( dpy ) != 0 )
    {
      XNextEvent ( dpy, & event );

      if ( event.type == ButtonPress )
      {
        if ( ! XQueryPointer ( dpy, root, & new_root, & child, & root_x, 
                           & root_y, & win_x, & win_y, & mask ) )
        {
          printf ( "Couldn't query pointer...\n" );
          return ( ( Window ) NULL );
        }

        XUngrabPointer ( dpy, CurrentTime );
        return ( child );
      }
    }
  }
}


/*---------------------------------------------------------------------*/
/*                                                                     */
/*---------------------------------------------------------------------*/
static Window * GetWindowChildren ( Display * dpy, Window win, int * numKids )
{
  Window root, parent, * kids;

  * numKids = 0 ;

  XQueryTree ( dpy, win, & root, & parent, &  kids, numKids );

  return ( kids );
}


/*-------------------------------------------------------------------------*/
/*                                                                         */
/*-------------------------------------------------------------------------*/
void ProcessKey ( char ch )
{
  KeySym keysym;
  int    shift;

  shift = 0;

  switch ( ch )
  {
    case '\n':
         keysym = XK_Return;
         break;

    case '?':
         shift = ShiftMask;
         break;

    default:
         if ( isupper ( ch ) )
         {
           shift = ShiftMask;
         }
         keysym = ( KeySym ) ch;
         break;
  }

  SendTheKey ( target_dpy, target_win, keysym, shift, KeyPress );
  SendTheKey ( target_dpy, target_win, keysym, shift, KeyRelease );
}


/*-------------------------------------------------------------------------*/
/*                                                                         */
/*-------------------------------------------------------------------------*/
void SendTheKey ( Display * display, Window window,
                  KeySym key, unsigned long shift_state, int type )
{
  XEvent   Event;
  int      i;
  int      x, y;
  Window   root, child;

  root = XDefaultRootWindow ( display );

  XTranslateCoordinates ( display, window, root, 0, 0, & x, & y, & child );

  Event.xkey.type = type;
  Event.xkey.serial = 1234;
  Event.xkey.send_event = FALSE;
  Event.xkey.display = display;
  Event.xkey.window = window;
  Event.xkey.subwindow = None;
  Event.xkey.root = DefaultRootWindow ( display );
  Event.xkey.time = CurrentTime;
  Event.xkey.x = 0;
  Event.xkey.y = 0;
  Event.xkey.x_root = x;
  Event.xkey.y_root = y;
/*?*/ Event.xkey.same_screen = TRUE;

  Event.xkey.state = shift_state;

  Event.xkey.keycode = XKeysymToKeycode ( display, key );

  XSendEvent ( display, window, True, KeyPressMask, & Event );
}

